@@ -1,7 +1,9 @@
 #define _POSIX_C_SOURCE 200809L
 #include <assert.h>
 #include <errno.h>
+#include <getopt.h>
 #include <limits.h>
+#include <signal.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
@@ -77,7 +79,9 @@ static struct kanshi_profile *match(struct kanshi_state *state,
 
 static void exec_command(char *cmd) {
 	pid_t pid, child;
+	// Fork process
 	if ((pid = fork()) == 0) {
+		// Fork child process again
 		setsid();
 		sigset_t set;
 		sigemptyset(&set);
@@ -92,26 +96,16 @@ static void exec_command(char *cmd) {
 					" command '%s': %s", cmd, strerror(errno));
 			exit(0);
 		}
-
-		// Try to give some meaningful information on the command success
-		int wstatus;
-		if (waitpid(child, &wstatus, 0) != child) {
-			perror("waitpid");
-			exit(0);
-		}
-		if (WIFEXITED(wstatus)) {
-			fprintf(stderr, "Command '%s' returned with exit status %d.\n",
-					cmd, WEXITSTATUS(wstatus));
-		} else {
-			fprintf(stderr, "Command '%s' was killed, aborted or disappeared"
-					" in dire circumstances.\n", cmd);
-		}
 		exit(0); // Close child process
 	}
 
 	if (pid < 0) {
 		perror("Impossible to fork a new process");
+		return;
 	}
+
+	// cleanup child process
+	waitpid(pid, NULL, 0);
 }
 
 static void execute_profile_commands(struct kanshi_profile *profile) {
@@ -460,7 +454,11 @@ static const struct wl_registry_listener registry_listener = {
 	.global_remove = registry_handle_global_remove,
 };
 
-static struct kanshi_config *read_config(void) {
+static struct kanshi_config *read_config(const char *config) {
+	if (config != NULL) {
+		return parse_config(config);
+	}
+
 	const char config_filename[] = "kanshi/config";
 	char config_path[PATH_MAX];
 	const char *xdg_config_home = getenv("XDG_CONFIG_HOME");
@@ -479,12 +477,42 @@ static struct kanshi_config *read_config(void) {
 	return parse_config(config_path);
 }
 
+static const char usage[] = "Usage: %s [options...]\n"
+"  -h, --help           Show help message and quit\n"
+"  -c, --config <path>  Path to config file.\n";
+
+static const struct option long_options[] = {
+	{"help", no_argument, 0, 'h'},
+	{"config", required_argument, 0, 'c'},
+	{0},
+};
+
 int main(int argc, char *argv[]) {
-	struct kanshi_config *config = read_config();
+	char *config_arg = NULL;
+
+	int opt;
+	while ((opt = getopt_long(argc, argv, "hc:", long_options, NULL)) != -1) {
+		switch (opt) {
+		case 'c':
+			free(config_arg);
+			config_arg = strdup(optarg);
+			break;
+		case 'h':
+			fprintf(stderr, usage, argv[0]);
+			return EXIT_SUCCESS;
+		default:
+			fprintf(stderr, usage, argv[0]);
+			return EXIT_FAILURE;
+		}
+	}
+
+	struct kanshi_config *config = read_config(config_arg);
 	if (config == NULL) {
 		return EXIT_FAILURE;
 	}
 
+	free(config_arg);
+
 	struct wl_display *display = wl_display_connect(NULL);
 	if (display == NULL) {
 		fprintf(stderr, "failed to connect to display\n");
